# flake8: noqa
# Countries list (ISO-3166)
COUNTRIES = {
    "##": "non renseign\xc3\xa9",
    "..": "non renseign\xc3\xa9",
    "aa": "aire g\xc3\xa9ographique ancienne",
    "ad": "Andorre",
    "ae": "\xc3\x89mirats arabes unis",
    "af": "Afghanistan",
    "ag": "Antigua-et-Barbuda",
    "ai": "Anguilla",
    "al": "Albanie",
    "am": "Arm\xc3\xa9nie",
    "an": "Antilles n\xc3\xa9erlandaises",
    "ao": "Angola",
    "aq": "Antarctique",
    "ar": "Argentine",
    "as": "Samoa am\xc3\xa9ricaines",
    "at": "Autriche",
    "au": "Australie",
    "aw": "Aruba",
    "ax": "Aland (\xc3\xaeles)",
    "az": "Azerba\xc3\xafdjan",
    "ba": "Bosnie-Herz\xc3\xa9govine",
    "bb": "Barbade",
    "bd": "Bangladesh",
    "be": "Belgique",
    "bf": "Burkina",
    "bg": "Bulgarie",
    "bh": "Bahre\xc3\xafn",
    "bi": "Burundi",
    "bj": "B\xc3\xa9nin",
    "bl": "Saint-Barth\xc3\xa9lemy",
    "bm": "Bermudes",
    "bn": "Brun\xc3\xa9i",
    "bo": "Bolivie",
    "bq": "Bonaire, Saint-Eustache et Saba,Saba###Saint-Eustache",
    "br": "Br\xc3\xa9sil",
    "bs": "Bahamas",
    "bt": "Bhoutan",
    "bv": "Bouvet (\xc3\xaele)",
    "bw": "Botswana",
    "by": "Bi\xc3\xa9lorussie,B\xc3\xa9larus",
    "bz": "Belize",
    "ca": "Canada",
    "cc": "Cocos (\xc3\xaeles),Keeling (\xc3\xaeles)",
    "cd": "Congo (R\xc3\xa9publique d\xc3\xa9mocratique),Za\xc3\xafre",
    "cf": "Centrafrique,R\xc3\xa9publique centrafricaine",
    "cg": "Congo,Congo (R\xc3\xa9publique)",
    "ch": "Suisse,Conf\xc3\xa9d\xc3\xa9ration helv\xc3\xa9tique",
    "ci": "C\xc3\xb4te d'Ivoire\n",
    "ck": "Cook (\xc3\xaeles)",
    "cl": "Chili",
    "cm": "Cameroun",
    "cn": "Chine,Chine (R\xc3\xa9publique populaire)",
    "co": "Colombie",
    "cr": "Costa Rica",
    "cs": "Serbie-et-Mont\xc3\xa9n\xc3\xa9gro",
    "cu": "Cuba",
    "cv": "Cap-Vert",
    "cw": "Cura\xc3\xa7ao",
    "cx": "Christmas (\xc3\xaele)",
    "cy": "Chypre",
    "cz": "R\xc3\xa9publique tch\xc3\xa8que,Tch\xc3\xa8que, R\xc3\xa9publique",
    "dd": "Allemagne (R\xc3\xa9publique d\xc3\xa9mocratique)",
    "de": "Allemagne,Allemagne (R\xc3\xa9publique f\xc3\xa9d\xc3\xa9rale)",
    "dj": "Djibouti",
    "dk": "Danemark",
    "dm": "Dominique",
    "do": "R\xc3\xa9publique dominicaine,Dominicaine, R\xc3\xa9publique",
    "dz": "Alg\xc3\xa9rie",
    "ec": "\xc3\x89quateur",
    "ee": "Estonie",
    "eg": "\xc3\x89gypte",
    "eh": "Sahara occidental",
    "er": "\xc3\x89rythr\xc3\xa9e",
    "es": "Espagne",
    "et": "\xc3\x89thiopie",
    "fi": "Finlande",
    "fj": "Fidji",
    "fk": "Malouines (\xc3\xaeles),Falkland (\xc3\xaeles)",
    "fm": "Micron\xc3\xa9sie,\xc3\x89tats f\xc3\xa9d\xc3\xa9r\xc3\xa9s de Micron\xc3\xa9sie",
    "fo": "F\xc3\xa9ro\xc3\xa9 (\xc3\xaeles)",
    "fr": "France",
    "ga": "Gabon",
    "gb": "Grande-Bretagne,Royaume-Uni",
    "gd": "Grenade",
    "ge": "G\xc3\xa9orgie",
    "gf": "Guyane fran\xc3\xa7aise",
    "gg": "Guernesey",
    "gh": "Ghana",
    "gi": "Gibraltar",
    "gl": "Groenland",
    "gm": "Gambie",
    "gn": "Guin\xc3\xa9e",
    "gp": "Guadeloupe",
    "gq": "Guin\xc3\xa9e \xc3\xa9quatoriale",
    "gr": "Gr\xc3\xa8ce",
    "gs": "G\xc3\xa9orgie du Sud et les \xc3\xaeles Sandwich du Sud",
    "gt": "Guatemala",
    "gu": "Guam",
    "gw": "Guin\xc3\xa9e-Bissau",
    "gy": "Guyana",
    "hk": "Hong Kong",
    "hm": "Heard (\xc3\xaele) et \xc3\xaeles McDonald",
    "hn": "Honduras",
    "hr": "Croatie",
    "ht": "Ha\xc3\xafti",
    "hu": "Hongrie",
    "id": "Indon\xc3\xa9sie",
    "ie": "Irlande",
    "ii": "intergouvernemental",
    "il": "Isra\xc3\xabl",
    "im": "\xc3\x8ele de Man,Man, \xc3\x8ele de",
    "in": "Inde",
    "io": "Territoire britannique de l'Oc\xc3\xa9an indien,Chagos (\xc3\xaeles)###Oc\xc3\xa9an indien, Territoire britannique de l'\n",
    "iq": "Irak",
    "ir": "Iran",
    "is": "Islande",
    "it": "Italie",
    "je": "Jersey",
    "jm": "Jama\xc3\xafque",
    "jo": "Jordanie",
    "jp": "Japon",
    "ke": "Kenya",
    "kg": "Kirghizistan",
    "kh": "Cambodge",
    "ki": "Kiribati",
    "km": "Comores",
    "kn": "Saint-Kitts-et-Nevis,Saint-Christophe-et-Nevis",
    "ko": "Kosovo",
    "kp": "Cor\xc3\xa9e (R\xc3\xa9publique populaire d\xc3\xa9mocratique),Cor\xc3\xa9e du Nord",
    "kr": "Cor\xc3\xa9e (R\xc3\xa9publique),Cor\xc3\xa9e du Sud",
    "kw": "Kowe\xc3\xaft",
    "ky": "Cayman,Ca\xc3\xafmanes, \xc3\x8eles###Ca\xc3\xafman (\xc3\xaeles)",
    "kz": "Kazakhstan",
    "la": "Laos",
    "lb": "Liban",
    "lc": "Sainte-Lucie",
    "li": "Liechtenstein",
    "lk": "Sri Lanka",
    "lr": "Liberia",
    "ls": "Lesotho",
    "lt": "Lituanie",
    "lu": "Luxembourg",
    "lv": "Lettonie",
    "ly": "Libye",
    "ma": "Maroc",
    "mc": "Monaco",
    "md": "Moldavie,Moldova, R\xc3\xa9publique de",
    "me": "Mont\xc3\xa9n\xc3\xa9gro",
    "mf": "Saint-Martin (partie fran\xc3\xa7aise)",
    "mg": "Madagascar",
    "mh": "Marshall (\xc3\xaeles)",
    "mk": "Mac\xc3\xa9doine (R\xc3\xa9publique)",
    "ml": "Mali",
    "mm": "Myanmar,Birmanie",
    "mn": "Mongolie",
    "mo": "Macao",
    "mp": "Mariannes du Nord (\xc3\xaeles)",
    "mq": "Martinique",
    "mr": "Mauritanie",
    "ms": "Montserrat",
    "mt": "Malte",
    "mu": "Maurice",
    "mv": "Maldives",
    "mw": "Malawi",
    "mx": "Mexique",
    "my": "Malaisie",
    "mz": "Mozambique",
    "na": "Namibie",
    "nc": "Nouvelle-Cal\xc3\xa9donie",
    "ne": "Niger",
    "nf": "Norfolk (\xc3\xaele)",
    "ng": "Nigeria",
    "ni": "Nicaragua",
    "nl": "Pays-Bas",
    "no": "Norv\xc3\xa8ge",
    "np": "N\xc3\xa9pal",
    "nr": "Nauru",
    "nu": "Niue",
    "nz": "Nouvelle-Z\xc3\xa9lande",
    "om": "Oman",
    "oo": "code non adapt\xc3\xa9",
    "pa": "Panama",
    "pe": "P\xc3\xa9rou",
    "pf": "Polyn\xc3\xa9sie fran\xc3\xa7aise",
    "pg": "Papouasie-Nouvelle-Guin\xc3\xa9e",
    "ph": "Philippines",
    "pk": "Pakistan",
    "pl": "Pologne",
    "pm": "Saint-Pierre-et-Miquelon",
    "pn": "Pitcairn",
    "pr": "Porto Rico",
    "ps": "Autorit\xc3\xa9 palestinienne,Palestine",
    "pt": "Portugal",
    "pw": "Palau,Palaos",
    "py": "Paraguay",
    "qa": "Qatar",
    "re": "R\xc3\xa9union",
    "ro": "Roumanie",
    "rs": "Serbie",
    "ru": "Russie (F\xc3\xa9d\xc3\xa9ration),Russie",
    "rw": "Rwanda",
    "sa": "Arabie saoudite",
    "sb": "Salomon (\xc3\xaeles)",
    "sc": "Seychelles",
    "sd": "Soudan",
    "se": "Su\xc3\xa8de",
    "sg": "Singapour",
    "sh": "Sainte-H\xc3\xa9l\xc3\xa8ne,Ascension (\xc3\xaele)###Tristan da Cunha (\xc3\xaele)",
    "si": "Slov\xc3\xa9nie",
    "sj": "Svalbard et \xc3\xaele Jan Mayen",
    "sk": "Slovaquie",
    "sl": "Sierra Leone",
    "sm": "Saint-Marin",
    "sn": "S\xc3\xa9n\xc3\xa9gal",
    "so": "Somalie",
    "sr": "Suriname",
    "ss": "Soudan du Sud,Sud Soudan",
    "st": "Sao Tom\xc3\xa9-et-Principe",
    "su": "URSS",
    "sv": "El Salvador,Salvador",
    "sx": "Saint-Martin (partie n\xc3\xa9erlandaise),Sint Maarten",
    "sy": "Syrie",
    "sz": "Swaziland",
    "tc": "Turks et Ca\xc3\xafques (\xc3\xaeles)",
    "td": "Tchad",
    "tf": "Terres australes fran\xc3\xa7aises",
    "tg": "Togo",
    "th": "Tha\xc3\xaflande",
    "tj": "Tadjikistan",
    "tk": "Tokelau",
    "tl": "Timor oriental",
    "tm": "Turkm\xc3\xa9nistan",
    "tn": "Tunisie",
    "to": "Tonga",
    "tr": "Turquie",
    "tt": "Trinit\xc3\xa9-et-Tobago",
    "tv": "Tuvalu",
    "tw": "Ta\xc3\xafwan,Chine (R\xc3\xa9publique)",
    "tz": "Tanzanie",
    "ua": "Ukraine",
    "ug": "Ouganda",
    "um": "\xc3\x8eles mineures \xc3\xa9loign\xc3\xa9es des \xc3\x89tats-Unis",
    "us": "\xc3\x89tats-Unis",
    "uy": "Uruguay",
    "uz": "Ouzb\xc3\xa9kistan",
    "va": "Vatican,Saint-Si\xc3\xa8ge",
    "vc": "Saint-Vincent-et-les Grenadines",
    "ve": "Venezuela",
    "vg": "\xc3\x8eles Vierges britanniques,Vierges (\xc3\xaeles) britanniques",
    "vi": "\xc3\x8eles Vierges am\xc3\xa9ricaines,Vierges (\xc3\xaeles) am\xc3\xa9ricaines",
    "vn": "Viet Nam",
    "vu": "Vanuatu",
    "wf": "Wallis et Futuna (\xc3\xaeles)",
    "ws": "Samoa,Samoa occidentales",
    "xc": "Tch\xc3\xa9coslovaquie",
    "xd": "Allemagne avant 1945",
    "xe": "Europe,Union europ\xc3\xa9enne",
    "xk": "Cor\xc3\xa9e avant 1948",
    "xn": "Pays-Bas avant 1830,Belgique avant 1830",
    "xx": "inconnu",
    "yd": "Y\xc3\xa9men (R\xc3\xa9publique d\xc3\xa9mocratique populaire),Sud Y\xc3\xa9men",
    "ye": "Y\xc3\xa9men",
    "yt": "Mayotte",
    "yu": "Yougoslavie",
    "yy": "ne s'applique pas\n",
    "za": "Afrique du Sud",
    "zm": "Zambie",
    "zw": "Zimbabwe",
    "zz": "multiple\n",
}


# REGIONS TO COUNTRIES MAPPING
REGIONS_TO_COUNTRIES = {
    "Abruzzes": "Italie",
    "Acha\xefe": "Gr\xe8ce",
    "Acre": "Br\xe9sil",
    "Afghanistan": "Afghanistan",
    "Afrique du Sud": "Afrique du Sud",
    "Aguascalientes": "Mexique",
    "Ain": "France",
    "Aisne": "France",
    "Alabama": "\xc9tats-Unis",
    "Alagoas": "Br\xe9sil",
    "Aland (\xeeles)": "Aland (\xeeles)",
    "Alaska": "\xc9tats-Unis",
    "Albanie": "Albanie",
    "Alberta": "Canada",
    "Alg\xe9rie": "Alg\xe9rie",
    "Allemagne": "Allemagne",
    "Allemagne (R\xe9publique d\xe9mocratique)": "Allemagne (R\xe9publique d\xe9mocratique)",
    "Allemagne avant 1945": "Allemagne avant 1945",
    "Allier": "France",
    "Alpes-Maritimes": "France",
    "Alpes-de-Haute-Provence": "France",
    "Alsace": "France",
    "Amapa": "Br\xe9sil",
    "Amazonas": "Br\xe9sil",
    "Andalousie": "Espagne",
    "Andorre": "Andorre",
    "Angola": "Angola",
    "Anguilla": "Anguilla",
    "Antarctique": "Antarctique",
    "Antigua-et-Barbuda": "Antigua-et-Barbuda",
    "Antilles n\xe9erlandaises": "Antilles n\xe9erlandaises",
    "Anvers": "Belgique",
    "Appenzell-Rhodes-Ext\xe9rieures": "Suisse",
    "Appenzell-Rhodes-Int\xe9rieures": "Suisse",
    "Aquitaine": "France",
    "Arabie saoudite": "Arabie saoudite",
    "Aragon": "Espagne",
    "Arcadie": "Gr\xe8ce",
    "Ardennes": "France",
    "Ard\xe8che": "France",
    "Argentine": "Argentine",
    "Argolide": "Gr\xe8ce",
    "Argovie": "Suisse",
    "Arizona": "\xc9tats-Unis",
    "Ari\xe8ge": "France",
    "Arkansas": "\xc9tats-Unis",
    "Arm\xe9nie": "Arm\xe9nie",
    "Aruba": "Aruba",
    "Asturies": "Espagne",
    "Ath\xe8nes et agglom\xe9ration": "Gr\xe8ce",
    "Attique": "Gr\xe8ce",
    "Aube": "France",
    "Aude": "France",
    "Australie": "Australie",
    "Australie-M\xe9ridionale": "Australie",
    "Australie-Occidentale": "Australie",
    "Autorit\xe9 palestinienne": "Autorit\xe9 palestinienne",
    "Autriche": "Autriche",
    "Auvergne": "France",
    "Aveyron": "France",
    "Azerba\xefdjan": "Azerba\xefdjan",
    "Bade-Wurtemberg": "Allemagne",
    "Bahamas": "Bahamas",
    "Bahia": "Br\xe9sil",
    "Bahre\xefn": "Bahre\xefn",
    "Baja California Norte": "Mexique",
    "Baja California Sur": "Mexique",
    "Bangladesh": "Bangladesh",
    "Barbade": "Barbade",
    "Bas-Rhin": "France",
    "Basilicate": "Italie",
    "Basse-Autriche": "Autriche",
    "Basse-Normandie": "France",
    "Basse-Saxe": "Allemagne",
    "Bavi\xe8re": "Allemagne",
    "Belgique": "Belgique",
    "Belize": "Belize",
    "Berlin": "Allemagne",
    "Bermudes": "Bermudes",
    "Berne": "Suisse",
    "Bhoutan": "Bhoutan",
    "Bi\xe9lorussie": "Bi\xe9lorussie",
    "Bolivie": "Bolivie",
    "Bonaire, Saint-Eustache et Saba": "Bonaire, Saint-Eustache et Saba",
    "Bosnie-Herz\xe9govine": "Bosnie-Herz\xe9govine",
    "Botswana": "Botswana",
    "Bouches-du-Rh\xf4ne": "France",
    "Bourgogne": "France",
    "Bouvet (\xeele)": "Bouvet (\xeele)",
    "Brabant": "Belgique",
    "Brabant flamand": "Belgique",
    "Brabant wallon": "Belgique",
    "Brabant-Septentrional": "Pays-Bas",
    "Brandebourg": "Allemagne",
    "Bretagne": "France",
    "Brun\xe9i": "Brun\xe9i",
    "Bruxelles": "Belgique",
    "Br\xe9sil": "Br\xe9sil",
    "Br\xeame": "Allemagne",
    "Buenos Aires": "Argentine",
    "Bulgarie": "Bulgarie",
    "Burgenland": "Autriche",
    "Burkina": "Burkina",
    "Burundi": "Burundi",
    "B\xe2le-Campagne": "Suisse",
    "B\xe2le-Ville": "Suisse",
    "B\xe9nin": "B\xe9nin",
    "B\xe9otie": "Gr\xe8ce",
    "Calabre": "Italie",
    "Californie": "\xc9tats-Unis",
    "Calvados": "France",
    "Cambodge": "Cambodge",
    "Cameroun": "Cameroun",
    "Campanie": "Italie",
    "Campeche": "Mexique",
    "Canada": "Canada",
    "Canaries": "Espagne",
    "Cantabrie": "Espagne",
    "Cantal": "France",
    "Cap-Vert": "Cap-Vert",
    "Capitale f\xe9d\xe9rale": "Argentine",
    "Carinthie": "Autriche",
    "Caroline du Nord": "\xc9tats-Unis",
    "Caroline du Sud": "\xc9tats-Unis",
    "Castille et L\xe9on": "Espagne",
    "Castille-la Manche": "Espagne",
    "Catalogne": "Espagne",
    "Catamarca": "Argentine",
    "Cayman": "Cayman",
    "Cear\xe1": "Br\xe9sil",
    "Centrafrique": "Centrafrique",
    "Centre": "France",
    "Ceuta": "Espagne",
    "Chaco": "Argentine",
    "Chalcidique": "Gr\xe8ce",
    "Champagne-Ardenne": "France",
    "Charente": "France",
    "Charente-Maritime": "France",
    "Cher": "France",
    "Chiapas": "Mexique",
    "Chihuahua": "Mexique",
    "Chili": "Chili",
    "Chine": "Chine",
    "Christmas (\xeele)": "Christmas (\xeele)",
    "Chubut": "Argentine",
    "Chypre": "Chypre",
    "Ch\xedos": "Gr\xe8ce",
    "Coahuila": "Mexique",
    "Cocos (\xeeles)": "Cocos (\xeeles)",
    "Colima": "Mexique",
    "Colombie": "Colombie",
    "Colombie britannique": "Canada",
    "Colorado": "\xc9tats-Unis",
    "Communaut\xe9 de Madrid": "Espagne",
    "Communaut\xe9 de Valence": "Espagne",
    "Comores": "Comores",
    "Congo": "Congo",
    "Congo (R\xe9publique d\xe9mocratique)": "Congo (R\xe9publique d\xe9mocratique)",
    "Connecticut": "\xc9tats-Unis",
    "Cook (\xeeles)": "Cook (\xeeles)",
    "Corfou": "Gr\xe8ce",
    "Corinthie": "Gr\xe8ce",
    "Corrientes": "Argentine",
    "Corr\xe8ze": "France",
    "Corse": "France",
    "Corse-du-Sud": "France",
    "Cor\xe9e (R\xe9publique populaire d\xe9mocratique)": "Cor\xe9e (R\xe9publique populaire d\xe9mocratique)",
    "Cor\xe9e (R\xe9publique)": "Cor\xe9e (R\xe9publique)",
    "Cor\xe9e avant 1948": "Cor\xe9e avant 1948",
    "Costa Rica": "Costa Rica",
    "Creuse": "France",
    "Croatie": "Croatie",
    "Cr\xe8te": "Gr\xe8ce",
    "Cuba": "Cuba",
    "Cura\xe7ao": "Cura\xe7ao",
    "Cyclades": "Gr\xe8ce",
    "C\xe9phalonie": "Gr\xe8ce",
    "C\xf3rdoba": "Argentine",
    "C\xf4te d'Ivoire": "C\xf4te d'Ivoire",
    "C\xf4te-d'Or": "France",
    "C\xf4tes-d'Armor": "France",
    "Dakota du Nord": "\xc9tats-Unis",
    "Dakota du Sud": "\xc9tats-Unis",
    "Danemark": "Danemark",
    "Delaware": "\xc9tats-Unis",
    "Deux-S\xe8vres": "France",
    "District de Columbia": "\xc9tats-Unis",
    "District f\xe9d\xe9ral": "Br\xe9sil",
    "Djibouti": "Djibouti",
    "Dod\xe9can\xe8se": "Gr\xe8ce",
    "Dominique": "Dominique",
    "Dordogne": "France",
    "Doubs": "France",
    "Drenthe": "Pays-Bas",
    "Dr\xe1ma": "Gr\xe8ce",
    "Dr\xf4me": "France",
    "Durango": "Mexique",
    "D\xe9pendance de Ross (Nouvelle-Z\xe9lande)": "Antarctique",
    "El Salvador": "El Salvador",
    "Entre-Rios": "Argentine",
    "Espagne": "Espagne",
    "Espirito Santo": "Br\xe9sil",
    "Essonne": "France",
    "Estonie": "Estonie",
    "Estr\xe9madure": "Espagne",
    "Eub\xe9e": "Gr\xe8ce",
    "Eure": "France",
    "Eure-et-Loir": "France",
    "Eurytanie": "Gr\xe8ce",
    "Fidji": "Fidji",
    "Finist\xe8re": "France",
    "Finlande": "Finlande",
    "Flandre occidentale": "Belgique",
    "Flandre orientale": "Belgique",
    "Floride": "\xc9tats-Unis",
    "Fl\xf3rina": "Gr\xe8ce",
    "Formosa": "Argentine",
    "France": "France",
    "Franche-Comt\xe9": "France",
    "Fribourg": "Suisse",
    "Frioul-V\xe9n\xe9tie-Julienne": "Italie",
    "Frise": "Pays-Bas",
    "F\xe9ro\xe9 (\xeeles)": "F\xe9ro\xe9 (\xeeles)",
    "Gabon": "Gabon",
    "Galice": "Espagne",
    "Gambie": "Gambie",
    "Gard": "France",
    "Gen\xe8ve": "Suisse",
    "Gers": "France",
    "Ghana": "Ghana",
    "Gibraltar": "Gibraltar",
    "Gironde": "France",
    "Glaris": "Suisse",
    "Goi\xe1s": "Br\xe9sil",
    "Grande-Bretagne": "Grande-Bretagne",
    "Grenade": "Grenade",
    "Greven\xe1": "Gr\xe8ce",
    "Grisons": "Suisse",
    "Groenland": "Groenland",
    "Groningue": "Pays-Bas",
    "Gr\xe8ce": "Gr\xe8ce",
    "Gr\xe8ce centrale": "Gr\xe8ce",
    "Gr\xe8ce occidentale": "Gr\xe8ce",
    "Guadeloupe": "Guadeloupe",
    "Guam": "Guam",
    "Guanajuato": "Mexique",
    "Guatemala": "Guatemala",
    "Gueldre": "Pays-Bas",
    "Guernesey": "Guernesey",
    "Guerrero": "Mexique",
    "Guin\xe9e": "Guin\xe9e",
    "Guin\xe9e \xe9quatoriale": "Guin\xe9e \xe9quatoriale",
    "Guin\xe9e-Bissau": "Guin\xe9e-Bissau",
    "Guyana": "Guyana",
    "Guyane fran\xe7aise": "Guyane fran\xe7aise",
    "G\xe9orgie": "\xc9tats-Unis",
    "G\xe9orgie du Sud et les \xeeles Sandwich du Sud": "G\xe9orgie du Sud et les \xeeles Sandwich du Sud",
    "Hainaut": "Belgique",
    "Hambourg": "Allemagne",
    "Haut-Rhin": "France",
    "Haute-Autriche": "Autriche",
    "Haute-Corse": "France",
    "Haute-Garonne": "France",
    "Haute-Loire": "France",
    "Haute-Marne": "France",
    "Haute-Normandie": "France",
    "Haute-Savoie": "France",
    "Haute-Sa\xf4ne": "France",
    "Haute-Vienne": "France",
    "Hautes-Alpes": "France",
    "Hautes-Pyr\xe9n\xe9es": "France",
    "Hauts-de-Seine": "France",
    "Hawaii": "\xc9tats-Unis",
    "Ha\xefti": "Ha\xefti",
    "Heard (\xeele) et \xeeles McDonald": "Heard (\xeele) et \xeeles McDonald",
    "Hesse": "Allemagne",
    "Hidalgo": "Mexique",
    "Hollande-M\xe9ridionale": "Pays-Bas",
    "Hollande-Septentrionale": "Pays-Bas",
    "Honduras": "Honduras",
    "Hong Kong": "Hong Kong",
    "Hongrie": "Hongrie",
    "H\xe9rault": "France",
    "Idaho": "\xc9tats-Unis",
    "Ille-et-Vilaine": "France",
    "Illinois": "\xc9tats-Unis",
    "Inde": "Inde",
    "Indiana": "\xc9tats-Unis",
    "Indon\xe9sie": "Indon\xe9sie",
    "Indre": "France",
    "Indre-et-Loire": "France",
    "Iowa": "\xc9tats-Unis",
    "Io\xe1nnina": "Gr\xe8ce",
    "Irak": "Irak",
    "Iran": "Iran",
    "Irlande": "Irlande",
    "Ir\xe1kleion": "Gr\xe8ce",
    "Islande": "Islande",
    "Isra\xebl": "Isra\xebl",
    "Is\xe8re": "France",
    "Italie": "Italie",
    "Jalisco": "Mexique",
    "Jama\xefque": "Jama\xefque",
    "Japon": "Japon",
    "Jersey": "Jersey",
    "Jordanie": "Jordanie",
    "Jujuy": "Argentine",
    "Jura": "France",
    "Kansas": "\xc9tats-Unis",
    "Kard\xedtsa": "Gr\xe8ce",
    "Kastori\xe1": "Gr\xe8ce",
    "Kav\xe1la": "Gr\xe8ce",
    "Kazakhstan": "Kazakhstan",
    "Kentucky": "\xc9tats-Unis",
    "Kenya": "Kenya",
    "Kilk\xeds": "Gr\xe8ce",
    "Kirghizistan": "Kirghizistan",
    "Kiribati": "Kiribati",
    "Kosovo": "Kosovo",
    "Kowe\xeft": "Kowe\xeft",
    "Koz\xe1ni": "Gr\xe8ce",
    "La Can\xe9e": "Gr\xe8ce",
    "Laconie": "Gr\xe8ce",
    "Landes": "France",
    "Languedoc-Roussillon": "France",
    "Laos": "Laos",
    "Las\xedthi": "Gr\xe8ce",
    "Latium": "Italie",
    "Le Pir\xe9e": "Gr\xe8ce",
    "Lesotho": "Lesotho",
    "Lettonie": "Lettonie",
    "Leucade": "Gr\xe8ce",
    "Liban": "Liban",
    "Liberia": "Liberia",
    "Libye": "Libye",
    "Liechtenstein": "Liechtenstein",
    "Ligurie": "Italie",
    "Limbourg": "Pays-Bas",
    "Limousin": "France",
    "Lituanie": "Lituanie",
    "Li\xe8ge": "Belgique",
    "Loir-et-Cher": "France",
    "Loire": "France",
    "Loire-Atlantique": "France",
    "Loiret": "France",
    "Lombardie": "Italie",
    "Lorraine": "France",
    "Lot": "France",
    "Lot-et-Garonne": "France",
    "Louisiane": "\xc9tats-Unis",
    "Loz\xe8re": "France",
    "Lucerne": "Suisse",
    "Luxembourg": "Belgique",
    "L\xe1risa": "Gr\xe8ce",
    "L\xe9svos": "Gr\xe8ce",
    "Macao": "Macao",
    "Mac\xe9doine (R\xe9publique)": "Mac\xe9doine (R\xe9publique)",
    "Mac\xe9doine centrale": "Gr\xe8ce",
    "Mac\xe9doine occidentale": "Gr\xe8ce",
    "Mac\xe9doine orientale et Thrace": "Gr\xe8ce",
    "Madagascar": "Madagascar",
    "Magn\xe9sie": "Gr\xe8ce",
    "Maine": "\xc9tats-Unis",
    "Maine-et-Loire": "France",
    "Malaisie": "Malaisie",
    "Malawi": "Malawi",
    "Maldives": "Maldives",
    "Mali": "Mali",
    "Malouines (\xeeles)": "Malouines (\xeeles)",
    "Malte": "Malte",
    "Manche": "France",
    "Manitoba": "Canada",
    "Maranh\xe3o": "Br\xe9sil",
    "Marches": "Italie",
    "Mariannes du Nord (\xeeles)": "Mariannes du Nord (\xeeles)",
    "Marne": "France",
    "Maroc": "Maroc",
    "Marshall (\xeeles)": "Marshall (\xeeles)",
    "Martinique": "Martinique",
    "Maryland": "\xc9tats-Unis",
    "Massachusetts": "\xc9tats-Unis",
    "Mato grosso": "Br\xe9sil",
    "Mato grosso do Sul": "Br\xe9sil",
    "Maurice": "Maurice",
    "Mauritanie": "Mauritanie",
    "Mayenne": "France",
    "Mayotte": "Mayotte",
    "Mecklembourg-Pom\xe9ranie ant\xe9rieure": "Allemagne",
    "Melilla": "Espagne",
    "Mendoza": "Argentine",
    "Mess\xe9nie": "Gr\xe8ce",
    "Meurthe-et-Moselle": "France",
    "Meuse": "France",
    "Mexico": "Mexique",
    "Mexique": "Mexique",
    "Michigan": "\xc9tats-Unis",
    "Michoac\xe1n": "Mexique",
    "Micron\xe9sie": "Micron\xe9sie",
    "Midi-Pyr\xe9n\xe9es": "France",
    "Minas Gerais": "Br\xe9sil",
    "Minnesota": "\xc9tats-Unis",
    "Misiones": "Argentine",
    "Mississippi": "\xc9tats-Unis",
    "Missouri": "\xc9tats-Unis",
    "Moldavie": "Moldavie",
    "Molise": "Italie",
    "Monaco": "Monaco",
    "Mongolie": "Mongolie",
    "Montana": "\xc9tats-Unis",
    "Montserrat": "Montserrat",
    "Mont\xe9n\xe9gro": "Mont\xe9n\xe9gro",
    "Morbihan": "France",
    "Morelos": "Mexique",
    "Moselle": "France",
    "Mozambique": "Mozambique",
    "Murcie": "Espagne",
    "Myanmar": "Myanmar",
    "Namibie": "Namibie",
    "Namur": "Belgique",
    "Nauru": "Nauru",
    "Navarre": "Espagne",
    "Nayarit": "Mexique",
    "Nebraska": "\xc9tats-Unis",
    "Neuch\xe2tel": "Suisse",
    "Neuqu\xe9n": "Argentine",
    "Nevada": "\xc9tats-Unis",
    "New Hampshire": "\xc9tats-Unis",
    "New Jersey": "\xc9tats-Unis",
    "New York": "\xc9tats-Unis",
    "Nicaragua": "Nicaragua",
    "Nidwald": "Suisse",
    "Niger": "Niger",
    "Nigeria": "Nigeria",
    "Niue": "Niue",
    "Ni\xe8vre": "France",
    "Nord": "France",
    "Nord-Pas-de-Calais": "France",
    "Norfolk (\xeele)": "Norfolk (\xeele)",
    "Norv\xe8ge": "Norv\xe8ge",
    "Nouveau Mexique": "\xc9tats-Unis",
    "Nouveau-Brunswick": "Canada",
    "Nouvelle-Cal\xe9donie": "Nouvelle-Cal\xe9donie",
    "Nouvelle-Galles-du-Sud": "Australie",
    "Nouvelle-Z\xe9lande": "Nouvelle-Z\xe9lande",
    "Nouvelle-\xc9cosse": "Canada",
    "Nuevo Le\xf3n": "Mexique",
    "N\xe9pal": "N\xe9pal",
    "Oaxaca": "Mexique",
    "Obwald": "Suisse",
    "Ohio": "\xc9tats-Unis",
    "Oise": "France",
    "Oklahoma": "\xc9tats-Unis",
    "Oman": "Oman",
    "Ombrie": "Italie",
    "Ontario": "Canada",
    "Oregon": "\xc9tats-Unis",
    "Orne": "France",
    "Ouganda": "Ouganda",
    "Ouzb\xe9kistan": "Ouzb\xe9kistan",
    "Overijssell": "Pays-Bas",
    "Pakistan": "Pakistan",
    "Palau": "Palau",
    "Pampa": "Argentine",
    "Panama": "Panama",
    "Papouasie-Nouvelle-Guin\xe9e": "Papouasie-Nouvelle-Guin\xe9e",
    "Paraguay": "Paraguay",
    "Paraiba": "Br\xe9sil",
    "Param\xe1": "Br\xe9sil",
    "Paris": "France",
    "Par\xe1": "Br\xe9sil",
    "Pas-de-Calais": "France",
    "Pays Basque": "Espagne",
    "Pays-Bas": "Pays-Bas",
    "Pays-Bas avant 1830": "Pays-Bas avant 1830",
    "Pays-de-la-Loire": "France",
    "Pennsylvanie": "\xc9tats-Unis",
    "Pernambouc": "Br\xe9sil",
    "Philippines": "Philippines",
    "Phocide": "Gr\xe8ce",
    "Phtiotide": "Gr\xe8ce",
    "Piau\xed": "Br\xe9sil",
    "Picardie": "France",
    "Pitcairn": "Pitcairn",
    "Pi\xe9mont": "Italie",
    "Pi\xe9rie": "Gr\xe8ce",
    "Poitou-Charentes": "France",
    "Pologne": "Pologne",
    "Polyn\xe9sie fran\xe7aise": "Polyn\xe9sie fran\xe7aise",
    "Porto Rico": "Porto Rico",
    "Portugal": "Portugal",
    "Pouilles": "Italie",
    "Provence-Alpes-C\xf4te d'Azur": "France",
    "Pr\xe9veza": "Gr\xe8ce",
    "Puebla": "Mexique",
    "Puy-de-D\xf4me": "France",
    "Pyr\xe9n\xe9es-Atlantiques": "France",
    "Pyr\xe9n\xe9es-Orientales": "France",
    "P\xe9lla": "Gr\xe8ce",
    "P\xe9loponn\xe8se": "Gr\xe8ce",
    "P\xe9rou": "P\xe9rou",
    "Qatar": "Qatar",
    "Queensland": "Australie",
    "Quer\xe9taro": "Mexique",
    "Quintana Roo": "Mexique",
    "Qu\xe9bec": "Canada",
    "Rhode Island": "\xc9tats-Unis",
    "Rhodope": "Gr\xe8ce",
    "Rh\xe9nanie-Palatinat": "Allemagne",
    "Rh\xe9nanie-du-Nord-Westphalie": "Allemagne",
    "Rh\xf4ne": "France",
    "Rh\xf4ne-Alpes": "France",
    "Rio Grande do Norte": "Br\xe9sil",
    "Rio Grande do Sul": "Br\xe9sil",
    "Rio Negro": "Argentine",
    "Rio de Janeiro": "Br\xe9sil",
    "Rioja": "Argentine",
    "Rond\xf4nia": "Br\xe9sil",
    "Roraima": "Br\xe9sil",
    "Roumanie": "Roumanie",
    "Royaume-Uni": "Grande-Bretagne",
    "Russie (F\xe9d\xe9ration)": "Russie (F\xe9d\xe9ration)",
    "Rwanda": "Rwanda",
    "R\xc3\xa9publique Tch\xc3\xa8que": "R\xc3\xa9publique tch\xc3\xa8que",
    "R\xe9publique dominicaine": "R\xe9publique dominicaine",
    "R\xe9publique tch\xe8que": "R\xe9publique tch\xe8que",
    "R\xe9thymnon": "Gr\xe8ce",
    "R\xe9union": "R\xe9union",
    "Sahara occidental": "Sahara occidental",
    "Saint-Barth\xe9lemy": "Saint-Barth\xe9lemy",
    "Saint-Gall": "Suisse",
    "Saint-Kitts-et-Nevis": "Saint-Kitts-et-Nevis",
    "Saint-Marin": "Saint-Marin",
    "Saint-Martin (partie fran\xe7aise)": "Saint-Martin (partie fran\xe7aise)",
    "Saint-Martin (partie n\xe9erlandaise)": "Saint-Martin (partie n\xe9erlandaise)",
    "Saint-Pierre-et-Miquelon": "Saint-Pierre-et-Miquelon",
    "Saint-Vincent-et-les Grenadines": "Saint-Vincent-et-les Grenadines",
    "Sainte-H\xe9l\xe8ne": "Sainte-H\xe9l\xe8ne",
    "Sainte-Lucie": "Sainte-Lucie",
    "Salomon (\xeeles)": "Salomon (\xeeles)",
    "Salta": "Argentine",
    "Salzbourg": "Autriche",
    "Samoa": "Samoa",
    "Samoa am\xe9ricaines": "Samoa am\xe9ricaines",
    "San Juan": "Argentine",
    "San Luis": "Argentine",
    "San Luis Potos\xed": "Mexique",
    "Santa Catarina": "Br\xe9sil",
    "Santa Cruz": "Argentine",
    "Santa Fe": "Argentine",
    "Santiago del Estero": "Argentine",
    "Sao Tom\xe9-et-Principe": "Sao Tom\xe9-et-Principe",
    "Sardaigne": "Italie",
    "Sarre": "Allemagne",
    "Sarthe": "France",
    "Saskatchewan": "Canada",
    "Savoie": "France",
    "Saxe": "Allemagne",
    "Saxe-Anhalt": "Allemagne",
    "Sa\xf4ne-et-Loire": "France",
    "Schaffhouse": "Suisse",
    "Schleswig-Holstein": "Allemagne",
    "Schwyz": "Suisse",
    "Seine-Maritime": "France",
    "Seine-Saint-Denis": "France",
    "Seine-et-Marne": "France",
    "Serbie": "Serbie",
    "Serbie-et-Mont\xe9n\xe9gro": "Serbie-et-Mont\xe9n\xe9gro",
    "Sergipe": "Br\xe9sil",
    "Seychelles": "Seychelles",
    "Sicile": "Italie",
    "Sierra Leone": "Sierra Leone",
    "Sinaloa": "Mexique",
    "Singapour": "Singapour",
    "Slovaquie": "Slovaquie",
    "Slov\xe9nie": "Slov\xe9nie",
    "Soleure": "Suisse",
    "Somalie": "Somalie",
    "Somme": "France",
    "Sonora": "Mexique",
    "Soudan": "Soudan",
    "Soudan du Sud": "Soudan du Sud",
    "Sri Lanka": "Sri Lanka",
    "Styrie": "Autriche",
    "Suisse": "Suisse",
    "Suriname": "Suriname",
    "Su\xe8de": "Su\xe8de",
    "Svalbard et \xeele Jan Mayen": "Svalbard et \xeele Jan Mayen",
    "Swaziland": "Swaziland",
    "Syrie": "Syrie",
    "S\xe1mos": "Gr\xe8ce",
    "S\xe3o Paulo": "Br\xe9sil",
    "S\xe9n\xe9gal": "S\xe9n\xe9gal",
    "S\xe9rrai": "Gr\xe8ce",
    "Tabasco": "Mexique",
    "Tadjikistan": "Tadjikistan",
    "Tamaulipas": "Mexique",
    "Tanzanie": "Tanzanie",
    "Tarn": "France",
    "Tarn-et-Garonne": "France",
    "Tasmanie": "Australie",
    "Ta\xefwan": "Ta\xefwan",
    "Tchad": "Tchad",
    "Tch\xe9coslovaquie": "Tch\xe9coslovaquie",
    "Tennessee": "\xc9tats-Unis",
    "Terre de Feu": "Argentine",
    "Terre de la Reine-Maud (Norv\xe8ge)": "Antarctique",
    "Terre-Neuve": "Canada",
    "Terres australes et antarctiques fran\xe7aises": "Antarctique",
    "Terres australes fran\xe7aises": "Terres australes fran\xe7aises",
    "Territoire antarctique australien": "Antarctique",
    "Territoire antarctique britannique": "Antarctique",
    "Territoire britannique de l'Oc\xe9an indien": "Territoire britannique de l'Oc\xe9an indien",
    "Territoire de la capitale australienne": "Australie",
    "Territoire du Nord": "Australie",
    "Territoire du Yukon": "Canada",
    "Territoire-de-Belfort": "France",
    "Territoires du Nord-Ouest": "Canada",
    "Tessin": "Suisse",
    "Texas": "\xc9tats-Unis",
    "Tha\xeflande": "Tha\xeflande",
    "Thesprotie": "Gr\xe8ce",
    "Thessalie": "Gr\xe8ce",
    "Thessalonique": "Gr\xe8ce",
    "Thurgovie": "Suisse",
    "Thuringe": "Allemagne",
    "Timor oriental": "Timor oriental",
    "Tlaxcala": "Mexique",
    "Togo": "Togo",
    "Tokelau": "Tokelau",
    "Tonga": "Tonga",
    "Toscane": "Italie",
    "Trentin-Haut-Adige": "Italie",
    "Trinit\xe9-et-Tobago": "Trinit\xe9-et-Tobago",
    "Tr\xedkala": "Gr\xe8ce",
    "Tucum\xe1n": "Argentine",
    "Tunisie": "Tunisie",
    "Turkm\xe9nistan": "Turkm\xe9nistan",
    "Turks et Ca\xefques (\xeeles)": "Turks et Ca\xefques (\xeeles)",
    "Turquie": "Turquie",
    "Tuvalu": "Tuvalu",
    "Tyrol": "Autriche",
    "URSS": "URSS",
    "US": "\xc9tats-Unis",
    "USA": "\xc9tats-Unis",
    "Ukraine": "Ukraine",
    "Uri": "Suisse",
    "Uruguay": "Uruguay",
    "Utah": "\xc9tats-Unis",
    "Utrecht": "Pays-Bas",
    "Val d'Aoste": "Italie",
    "Val-d'Oise": "France",
    "Val-de-Marne": "France",
    "Valais": "Suisse",
    "Vanuatu": "Vanuatu",
    "Var": "France",
    "Vatican": "Vatican",
    "Vaucluse": "France",
    "Vaud": "Suisse",
    "Vend\xe9e": "France",
    "Venezuela": "Venezuela",
    "Veracruz": "Mexique",
    "Vermont": "\xc9tats-Unis",
    "Victoria": "Australie",
    "Vienne": "Autriche",
    "Viet Nam": "Viet Nam",
    "Virginie": "\xc9tats-Unis",
    "Virginie occidentale": "\xc9tats-Unis",
    "Vorarlberg": "Autriche",
    "Vosges": "France",
    "V\xe9n\xe9tie": "Italie",
    "Wallis et Futuna (\xeeles)": "Wallis et Futuna (\xeeles)",
    "Washington": "\xc9tats-Unis",
    "Wisconsin": "\xc9tats-Unis",
    "Wyoming": "\xc9tats-Unis",
    "X\xe1nthi": "Gr\xe8ce",
    "Yonne": "France",
    "Yougoslavie": "Yougoslavie",
    "Yucat\xe1n": "Mexique",
    "Yvelines": "France",
    "Y\xe9men": "Y\xe9men",
    "Y\xe9men (R\xe9publique d\xe9mocratique populaire)": "Y\xe9men (R\xe9publique d\xe9mocratique populaire)",
    "Zacatecas": "Mexique",
    "Zambie": "Zambie",
    "Zimbabwe": "Zimbabwe",
    "Zoug": "Suisse",
    "Zurich": "Suisse",
    "Z\xe1kynthos": "Gr\xe8ce",
    "Z\xe9lande": "Pays-Bas",
    "aire g\xe9ographique ancienne": "aire g\xe9ographique ancienne",
    "code non adapt\xe9": "code non adapt\xe9",
    "inconnu": "inconnu",
    "intergouvernemental": "intergouvernemental",
    "multiple": "multiple",
    "ne s'applique pas": "ne s'applique pas",
    "non renseign\xe9": "non renseign\xe9",
    "\xc1rta": "Gr\xe8ce",
    "\xc9gypte": "\xc9gypte",
    "\xc9lide": "Gr\xe8ce",
    "\xc9mathie": "Gr\xe8ce",
    "\xc9milie-Romagne": "Italie",
    "\xc9mirats arabes unis": "\xc9mirats arabes unis",
    "\xc9pire": "Gr\xe8ce",
    "\xc9quateur": "\xc9quateur",
    "\xc9rythr\xe9e": "\xc9rythr\xe9e",
    "\xc9tats-Unis": "\xc9tats-Unis",
    "\xc9thiopie": "\xc9thiopie",
    "\xc9tolie-et-Acarnanie": "Gr\xe8ce",
    "\xc9vros": "Gr\xe8ce",
    "\xcele Pierre 1er (Norv\xe8ge)": "Antarctique",
    "\xcele de Man": "\xcele de Man",
    "\xcele du Prince-\xc9douard": "Canada",
    "\xcele-de-France": "France",
    "\xceles Bal\xe9ares": "Espagne",
    "\xceles Ioniennes": "Gr\xe8ce",
    "\xceles Vierges am\xe9ricaines": "\xceles Vierges am\xe9ricaines",
    "\xceles Vierges britanniques": "\xceles Vierges britanniques",
    "\xceles de la Mer \xc9g\xe9e m\xe9ridionale": "Gr\xe8ce",
    "\xceles de la Mer \xc9g\xe9e septentrionale": "Gr\xe8ce",
    "\xceles mineures \xe9loign\xe9es des \xc9tats-Unis": "\xceles mineures \xe9loign\xe9es des \xc9tats-Unis",
}
